package org.apache.xindice.core;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: Database.java,v 1.2 2002/02/26 07:10:09 kstaken Exp $
 */

import org.apache.xindice.core.query.*;
import org.apache.xindice.core.security.*;
import org.apache.xindice.core.objects.*;
import org.apache.xindice.core.request.*;
import org.apache.xindice.util.*;

import org.w3c.dom.*;

import java.io.*;
import java.util.*;
import javax.xml.parsers.*;

/**
 * Database is the primary container for the Xindice Database Engine.
 */

public final class Database extends Collection implements Named {
   private static final String DBROOT = "dbroot";
   private static final String SECURITY = "security";
   private static final String PACKAGE = "package";
   private static final String QUERYENGINE = "queryengine";
   private static final String DATABASE = "database";
   private static final String COLKEY = "database.xml";
   private static final String NAME = "name";

   public static final String PROP_XINDICE_HOME = "xindice.home";
   
   private static final Map databases = new HashMap(); // String to Database

   private DocumentCache docCache = new DocumentCache();
   private DBSecurityManager securityManager = null;
   
   private ObjectPool uriPool = new ObjectPool() {
      public Poolable createObject() {
         return new URIMapper();
      }
   };

   private SystemCollection sysCol = null;

   private boolean sysInit = false;

   private QueryEngine engine = new QueryEngine(this);

   public Database() {
   }

   public static Database getDatabase(String name) {
      return (Database)databases.get(name);
   }

   public static String[] listDatabases() {
      return (String[]) databases.keySet().toArray(new String[0]);
   }

   public void setConfig(Configuration config) {
      this.config = config;

      name = config.getAttribute(NAME);
      setCanonicalName('/' + getName());
      
      String dbroot = config.getAttribute(DBROOT);
      if ( ! dbroot.startsWith("/") ) {
         dbroot = System.getProperty( PROP_XINDICE_HOME ) +
            "/" + dbroot;
      }
      setCollectionRoot(new File(dbroot));

      // Create the security manager so that it exists for loading the system
      // collections.
      // TODO: Finish security manager implementation
      if (securityManager == null) {
         boolean disableSecurity = false;
         if (config.getAttribute(SECURITY).equals("false")) {
            disableSecurity = true;
         }

         securityManager = new LocalSecurityManager(this, disableSecurity);
      }

      try {
         Configuration queryCfg = config.getChild(QUERYENGINE);
         if ( queryCfg != null )
            engine.setConfig(queryCfg);
      }
      catch ( Exception e ) {
         org.apache.xindice.Debug.printStackTrace(e);
      }

      if ( !sysInit ) {
         sysCol = new SystemCollection(this);

         try {
            sysCol.init();
         }
         catch ( XindiceException e ) {
            org.apache.xindice.Debug.printStackTrace(e);
         }

         collections.put(sysCol.getName(), sysCol);
         sysInit = true;
      }

      // System collections are loaded so we need to load the security config
      // and setup the system for secure operation.
      // TODO: Finish security manager implementation
      /*if (securityManager != null) {
         securityManager.readConfig();
         // Default admin level for the primary thread for testing.
         try {
            securityManager.authenticate("scott", "tiger");
         }
         catch (Exception e) {
            org.apache.xindice.Debug.printStackTrace(e);
         }
         securityManager.setActive();
      }*/

      
      try {
         // Bootstrap from the database itself...  This is accomplished
         // by intercepting the setConfig call and using a Configuration
         // retrieved from the database instead of the standard config
         Document colDoc = sysCol.getCollection(SystemCollection.CONFIGS).getDocument(COLKEY);
         if ( colDoc == null ) {
            DocumentBuilder db = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            colDoc = db.newDocument();
            Element root = colDoc.createElement(DATABASE);
            root.setAttribute(NAME, name);
            colDoc.appendChild(root);
            sysCol.getCollection(SystemCollection.CONFIGS).setDocument(COLKEY, colDoc);
         }
         
         super.setConfig(new Configuration(colDoc.getDocumentElement(), false));
      }
      catch ( Exception e ) {
         org.apache.xindice.Debug.printStackTrace(e);
      }

      // Register the Database with the VM
      databases.put(getName(), this);
   }

   public SystemCollection getSystemCollection() {
      return sysCol;
   }

   public Database getDatabase() {
      return this;
   }

   /**
    * flushConfig ensures that the Collection configuration has been
    * properly flushed to disk after a modification.
    */
   public void flushConfig() {
      try {
         Document d = config.getElement().getOwnerDocument();
         sysCol.getCollection(SystemCollection.CONFIGS).setDocument(COLKEY, d);
      }
      catch ( Exception e ) {
         org.apache.xindice.Debug.println(this, "Error Writing Configuration '"+name+"'");
      }
   }
  
   public boolean close() throws DBException {
      flushConfig();
      return true;
   }
      
   /**
    * getURIMapper returns a pooled URIMapper instance.
    *
    * @return A Pooled URIMapper
    */
   public URIMapper getURIMapper() {
      return (URIMapper)uriPool.getObject();
   }

   /**
    * getURIMapper returns a pooled URIMapper instance.
    *
    * @param uri The URI to resolve
    * @return A Pooled (and resolved) URIMapper
    */
   public URIMapper getURIMapper(String uri) throws DBException {
      try {
         URIMapper map = (URIMapper)uriPool.getObject();
         map.setURI(uri);
         return map;
      }
      catch ( Exception e ) {
         throw new DBException(FaultCodes.URI_PARSE_ERROR, "Couldn't Parse URL: '"+uri+"'");
      }
   }

   /**
    * getDocumentCache returns the Database-level Document Cache.
    *
    * @return The DocumentCache
    */
   public DocumentCache getDocumentCache() {
      return docCache;
   }

   /**
    * getQueryEngine returns a reference to the Database's current
    * operating QueryEngine implementation.
    *
    * @return The QueryEngine instance
    */
   public QueryEngine getQueryEngine() {
      return engine;
   }

   /**
    * Returns the security manager instance for this Database.
    *
    * @return The security manager
    */
   public DBSecurityManager getSecurityManager() {
      return securityManager;
   }
}

