package org.apache.xindice.server;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: Gateway.java,v 1.1.1.1 2001/12/06 19:33:55 bradford Exp $
 */

import org.apache.xindice.util.*;
import java.io.*;
import java.util.*;

/**
 * Gateway is an abstract foundation class used in the transfer of
 * protocol-independent data to and from a Xindice Script.
 * Gateway provides most of the functionality required by custom
 * implementations and does not hide the fact that it is partial to the HTTP
 * protocol. Gateway handles the construction of headers, cookies, and
 * persistent state objects. It also acts as a functional overlay to the public
 * methods of Kernel that can be exposed to a Script.
 * <br><br>
 * <strong>Note:</strong> The Gateway is a volatile object, usually
 * cached and reused by a Service after a request has been fulfilled.
 * You should not pass Gateway instances to methods that may refer to
 * those instances after the current request has been fulfilled because the
 * state of the Gateway will likely no longer be valid.
 *
 * @see org.apache.xindice.server.Script
 */

public abstract class Gateway {
   private static final VersionProxy SessionsProxy = new VersionProxy();
   private static String StartupTime = '-'+Long.toHexString(System.currentTimeMillis()).toUpperCase();

   private static ThreadLocal CurrentGateway = new ThreadLocal();

   protected Kernel kernel = null;

   protected Map reqHdr = new HashMap();  // Client Request Environment
   protected Map resHdr = new HashMap();  // Working Response Header
   protected Map cookies = new HashMap(); // Client/Working Cookies
   protected Map values = new HashMap();  // Form Values

   private String hexHashCode = Integer.toHexString(hashCode()).toUpperCase()+StartupTime;

   /**
    * getCurrentGateway returns the operating Gateway Object for this Thread.
    *
    * @return The Thread's Gateway
    */
   public static Gateway getCurrentGateway() {
      return (Gateway)CurrentGateway.get();
   }

   /**
    * setCurrentGateway sets the operating Gateway Object for this Thread.
    *
    * @param gateway The Thread's Gateway
    */
   protected static void setCurrentGateway(Gateway gateway) {
      CurrentGateway.set(gateway);
   }

   protected void init() {
      reqHdr.clear();
      resHdr.clear();
      cookies.clear();
      values.clear();
   }

   protected void setKernel(Kernel kernel) {
      this.kernel = kernel;
   }

   /**
    * Returns version information on the Gateway type used
    *
    * @return The gateway type (ex: "CGI1.0")
    */
   public abstract String getGatewayType();

   /* Abstract Input/Output methods */

   /* Standard Input and Output Stream methods */

   public abstract OutputStream getOutputStream();
   public abstract void resetOutputStream();
   public abstract OutputStream getNativeOutputStream();
   public abstract InputStream getInputStream();

   /* Buffered Input and Output methods */

   public abstract ByteBuffer getContent();
   public abstract ByteBuffer getHeaderContent(int code);
   public abstract void send(int code);
   public abstract void send();

   /* Error reporting methods */

   public abstract void sendError(int code, String message);
   public abstract void sendError(int code);
   public abstract void sendError();
   public abstract void sendException(String name, Exception e);

   /* Redirection methods */

   public abstract void redirectLocal(String URI);
   public abstract void redirectRemote(String URL);

   /* Path and Script name related methods */

   public abstract String getRootURL();     // Ex: http://cherub
   public abstract String getScriptName();  // Script Name (Class.forName())
   public abstract String getScriptPath();  // Extra Path Information
   public abstract String getScriptBase();  // Base Directory For Script
   public abstract String getPathAlias();   // Alias Name Received By Service

   /* Stub Methods To The API Managers */

   /* FileManager stubs */

   /**
    * getFileResource returns a FileResource instance for the
    * specified filename.
    *
    * @param filename The filename to retrieve
    * @return the FileResource
    */
   public FileResource getFileResource(String filename) {
      return kernel.getFileResource(filename);
   }

   /**
    * getTemporaryFileResource returns a temporary
    * FileResource instance.
    *
    * @return the FileResource
    */
   public FileResource getTemporaryFileResource() {
      return kernel.getTemporaryFileResource();
   }

   /* ResourceManager stubs */

   /**
    * setResource places a resource by name into the
    * ResourceManager.  The resource will be placed into the name
    * space for the provided Versioning object.
    *
    * @param versioning the version space to use
    * @param name the resource name
    * @param object the object resource
    * @throws ParameterException if the object cannot be stored
    */
   public void setResource(Versioning versioning, String name, Object object) throws ParameterException {
      kernel.setResource(versioning, name, object);
   }

   /**
    * setResource places a resource by name into the
    * ResourceManager. The resource will be placed into the default
    * name space.
    *
    * @param name the resource name
    * @param object the object resource
    * @throws ParameterException if the object cannot be stored
    */
   public void setResource(String name, Object object) throws ParameterException {
      kernel.setResource(name, object);
   }

   /**
    * removeResource removes a resource by name from the
    * ResourceManager.  The resource will be removed from the name
    * space for the provided Versioning object.
    *
    * @param versioning the version space to use
    * @param name the resource name
    */
   public void removeResource(Versioning versioning, String name) {
      kernel.removeResource(versioning, name);
   }

   /**
    * removeResource removes a resource by name from the
    * ResourceManager.  The resource will be removed from the
    * default name space.
    *
    * @param name the resource name
    */
   public void removeResource(String name) {
      kernel.removeResource(name);
   }

   /**
    * getResource retrieves a resource by name from the
    * ResourceManager.  The resource will be retrived from the name
    * space for the provided Versioning object.
    *
    * @param versioning the version space to use
    * @param name the resource name
    * @return the object resource
    */
   public Object getResource(Versioning versioning, String name) {
      return kernel.getResource(versioning, name);
   }

   /**
    * getResource retrieves a resource by name from the
    * ResourceManager.  The resource will be retrived from the
    * default name space.
    *
    * @param name the resource name
    * @return the object resource
    */
   public Object getResource(String name) {
      return kernel.getResource(name);
   }

   /**
    * runScript executs a Script in the
    * ScriptManager's storage by name.
    *
    * @param name the Script name
    */
   public void runScript(String name) {
      Script script = kernel.getScript(name);
      if ( script != null )
         kernel.runScript(script, this, false);
   }

   /* LogManager stubs */

   /**
    * logMessage logs a message to the LogManager.
    *
    * @param type The message type.
    * @param message The message
    */
   public void logMessage(int type, String message) {
      kernel.logMessage(type, getRequestHeader("REMOTE_HOST"), getRequestHeader("REMOTE_USER"), message);
   }

   /* ComponentManager Stubs */

   /**
    * getComponent returns a component of the specified name.
    *
    * @param name The name of the ScriptComponent
    * @return the ScriptComponent
    */
   public ScriptComponent getComponent(String name) {
      return kernel.getComponent(name);
   }

   /* Persistent state methods */

   protected String getSessionKey() {
      String value = getCookie("SessionKey");
      if ( value.length() == 0 || !value.endsWith(StartupTime) ) {
         value = Long.toHexString(System.currentTimeMillis()).toUpperCase()+hexHashCode;
         setCookie("SessionKey", value);
      }
      return value;
   }

   public Iterator enumObjects() {
      String skey = getSessionKey();
      Map session = (Map)getResource(SessionsProxy, skey);
      if ( session == null ) {
         try {
            setResource(SessionsProxy, skey, session = new HashMap());
         }
         catch ( Exception e ) {
         }
      }
      return session.keySet().iterator();
   }

   public Object getObject(String name) {
      Map session = (Map)getResource(SessionsProxy, getSessionKey());
      return session != null ? session.get(name)
                             : null;
   }

   public boolean isObjectNull(String name) {
      return getObject(name) == null;
   }

   public void setObject(String name, Object object) {
      String skey = getSessionKey();
      Map session = (Map)getResource(SessionsProxy, skey);
      if ( session == null ) {
         try {
            setResource(SessionsProxy, skey, session = new HashMap());
         }
         catch ( Exception e ) {
         }
      }
      session.put(name, object);
   }

   public void removeObject(String name) {
      Map session = (Map)getResource(SessionsProxy, getSessionKey());
      if ( session != null )
         session.remove(name);
   }

   public void removeAllObjects() {
      removeResource(SessionsProxy, getSessionKey());
   }

   /* Gateway Specific Methods */

   /* Client header methods */

   public Iterator enumRequestHeader() {
      return reqHdr.keySet().iterator();
   }

   public void setRequestHeader(String key, String value) {
      reqHdr.put(key.toUpperCase(), value);
   }

   public String getRequestHeader(String key) {
      String res = (String)reqHdr.get(key.toUpperCase());
      return res != null ? res : "";
   }

   public boolean isRequestHeaderNull(String key) {
      return reqHdr.get(key.toUpperCase()) == null;
   }

   /*  Working header methods */

   public Iterator enumResponseHeader() {
      return resHdr.keySet().iterator();
   }

   public void setResponseHeader(String key, String value) {
      resHdr.put(key, value);
   }

   public String getResponseHeader(String key) {
      String res = (String)resHdr.get(key);
      return res != null ? res : "";
   }

   public boolean isResponseHeaderNull(String key) {
      return resHdr.get(key) == null;
   }

   public void removeResponseHeader(String key) {
      resHdr.remove(key);
   }

   /* Cookie methods */

   public Iterator enumCookies() {
      return cookies.keySet().iterator();
   }

   public String getCookie(String key) {
      String res = (String)cookies.get(key);
      return res != null ? res : "";
   }

   public boolean isCookieNull(String key) {
      return cookies.get(key) == null;
   }

   public void removeCookie(String key) {
      resHdr.remove("@"+key);
      cookies.remove(key);
   }

   public void setCookie(String key, String value, Date expires, String path, String domain, boolean secure) {
      StringBuffer val = new StringBuffer(value.length()+64);
      val.append(value);
      if ( expires != null ) {
         val.append("; expires=");
         val.append(DateUtilities.getGMTString(expires));
      }
      val.append("; path=");
      val.append(path != null ? path : "/");
      if ( domain != null ) {
         val.append("; domain=");
         val.append(domain);
      }
      if ( secure )
         val.append("; secure");
      resHdr.put("@"+key, val.toString());
      cookies.put(key, value);
   }

   public void setCookie(String key, String value, Date expires, String path, String domain) {
      setCookie(key, value, expires, path, domain, false);
   }

   public void setCookie(String key, String value, Date expires, String path) {
      setCookie(key, value, expires, path, null);
   }

   public void setCookie(String key, String value, Date expires) {
      setCookie(key, value, expires, null);
   }

   public void setCookie(String key, String value) {
      setCookie(key, value, null);
   }

   /* User related methods */

   public User getUser() {
      String username = getRequestHeader("REMOTE_USER");
      return username.length() > 0 ? kernel.getUser(username)
                                   : null;
   }

   /* Form related methods */

   public Iterator enumFormValues() {
      return values.keySet().iterator();
   }

   public String getFormValue(String key) {
      String res = (String)values.get(key);
      return res != null ? res : "";
   }

   public boolean isFormValueNull(String key) {
      return values.get(key) == null;
   }

   public void setFormValue(String key, String value) {
      values.put(key, value);
   }

   public void removeFormValue(String key) {
      values.remove(key);
   }
}
