package org.apache.xindice.server.standard;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: StdLogManager.java,v 1.2 2002/02/26 07:10:09 kstaken Exp $
 */

import org.apache.xindice.server.*;
import org.apache.xindice.util.*;
import java.io.*;
import java.util.*;
import java.text.*;

/**
 * StdLogManager
 */

public final class StdLogManager implements Configurable, LogManager, KernelAccess, Disposable, Task {
   private static final DateFormatSymbols DFS = new DateFormatSymbols(Locale.US);
   static {
      DFS.setShortMonths(new String[] {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"});
      DFS.setShortWeekdays(new String[] {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"});
   }

   private static final SimpleDateFormat SDF = new SimpleDateFormat("MMM-dd-yyyy kk:mm:ss", DFS);
   
   private static final String FILENAME = "filename";
   private static final String ROTATE = "rotate";

   private static final String[] messages = {"", "WARNING", "EXCEPTION", "ERROR", "FATAL"};
   private static final long sleep = 1000;

   private Kernel kernel;
   private Configuration config;
   private String logname;
   private ByteBuffer buffer = new ByteBuffer(512);
   private ObjectQueue queue = new ObjectQueue(); // of StdLogEntry
   private long today = 0;
   private BufferedOutputStream log;
   private boolean archive = false;

   public void setKernel(Kernel kernel) {
      this.kernel = kernel;
   }

   public void setConfig(Configuration config) {
      this.config = config;
      logname = config.getAttribute(FILENAME);
      if ( ! logname.startsWith("/") ) {
         logname = System.getProperty( Xindice.PROP_XINDICE_HOME ) +
            "/" + config.getAttribute(FILENAME);    
      }
      
      archive = config.getBooleanAttribute(ROTATE);
      today = DateUtilities.getDateLong(new Date());
      if ( logname.length() > 0 ) {
         try {
            log = new BufferedOutputStream(new FileOutputStream(logname, true), 2048);
         }
         catch ( Exception e ) {
            System.err.println("\u0007ERROR: Couldn't create log file '"+logname+"'");
         }
      }
      kernel.addTask(this, 1000);
   }

   public Configuration getConfig() {
      return config;
   }

   public void logMessage(int type, String host, String user, String message) {
      writeLogEntry(type, host, user, message);
   }
   
   public void logMessage(int type, String host, String message) {
      writeLogEntry(type, host, "", message);
   }
   
   public void logMessage(int type, String message) {
      writeLogEntry(type, "0.0.0.0", "", message);
   }

   private void writeLogEntry(int type, String addr, String user, String message) {
      queue.add(new StdLogEntry(type, addr, user, message));
   }

   public void runTask() {
      if ( log == null ) // Then We Have No Business Being Here
         return;
      if ( !queue.isEmpty() ) {
         StdLogEntry entry;
         Date timestamp;
         do {
            if ( (entry = (StdLogEntry)queue.remove()) != null ) {
               try {
                  timestamp = new Date(entry.stamp);
                  if ( archive && DateUtilities.getDateLong(timestamp) != today ) {
                     // We Have To Archive The Old Log
                     log.close();
                     new File(logname).renameTo(new File(logname+"."+today));
                     log = new BufferedOutputStream(new FileOutputStream(logname, true), 2048);
                     today = DateUtilities.getDateLong(timestamp);
                  }
                  buffer.append(entry.addr);
                  buffer.append(" - ");
                  buffer.append(entry.user);
                  buffer.append(" - ");
                  buffer.append(SDF.format(timestamp));
                  buffer.append(" - ");
                  if ( entry.type >= 0 && entry.type < messages.length )
                     buffer.append(messages[entry.type]);
                  else
                     buffer.append("UNKNOWN");
                  buffer.append(" - ");
                  buffer.append(entry.message);
                  buffer.append("\r\n");
                  buffer.writeTo(log);
                  buffer.reset();
               }
               catch ( Exception e ) {
                  System.err.println("\u0007ERROR: "+e);
               }
            }
            //Thread.yield();
         }
         while ( !queue.isEmpty() );
         try {
            log.flush();
         }
         catch ( Exception e ) {
         }
      }
   }

   public void dispose() {
      kernel.removeTask(this);
      runTask();
      if ( log != null )
         try {
            log.close();
         }
         catch ( Exception e ) {
         }
   }

   /**
    * StdLogEntry
    */

   private class StdLogEntry {
      public long stamp = System.currentTimeMillis();
      public int type = 0;
      public String addr;
      public String user;
      public String message;

      public StdLogEntry(int type, String addr, String user, String message) {
         this.type = type;
         this.addr = addr;
         this.user = user;
         this.message = message;
      }
   }
}

