package org.apache.xindice.server.standard;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: StdServiceManager.java,v 1.1.1.1 2001/12/06 19:33:56 bradford Exp $
 */

import org.apache.xindice.server.*;
import org.apache.xindice.util.*;
import org.apache.xindice.server.services.*;
import java.io.*;
import java.util.*;

/**
 * StdServiceManager
 */

public final class StdServiceManager implements Configurable, ServiceManager, KernelAccess, Disposable {
   private static final String[] EmptyStrings = new String[0];
   private static final Service[] EmptyServices = new Service[0];
   
   private static final String SERVICE = "service";
   private static final String CLASS = "class";
   
   private Kernel kernel;
   private Configuration config;
   private List services = Collections.synchronizedList(new ArrayList()); // of Service
   private Map svcMap = Collections.synchronizedMap(new HashMap());

   public void setKernel(Kernel kernel) {
      this.kernel = kernel;
   }

   public void setConfig(Configuration config) {
      this.config = config;
      config.processChildren(SERVICE,
         new ConfigurationCallback() {
            public void process(Configuration cfg) {
               String className = cfg.getAttribute(CLASS);
               try {
                  Object obj = Class.forName(className).newInstance();
                  if ( obj instanceof Service ) {
                     Service service = (Service)obj;
                     kernel.setKernelOf(service);
                     
                     if ( obj instanceof Configurable ) {
                        ((Configurable)obj).setConfig(cfg);
                     }
                     
                     addService(service);
                  }
                  else {
                     System.err.println("\u0007ERROR: '"+className+"' is not a service");
                  }
               }
               catch ( Exception e ) {
                  System.err.println("\u0007FATAL ERROR: Loading service '" +className + "'" + e);
                  System.exit(1);
               }
            }
      });
   }
   
   public Configuration getConfig() {
      return config;
   }
   
   public boolean startServices() {
      boolean ok = true;
      Service[] svc = (Service[])services.toArray(EmptyServices);
      for ( int i = 0; i < svc.length; i++ ) {
         String name = svc[i].getName();
         if ( svc[i] instanceof ManagerService ) {
            ((ManagerService)svc[i]).setServiceManager(this);
         }
         
         if ( svc[i].start() == Service.RESULT_OK ) {
            if ( svc[i] instanceof SocketServer ) {
               SocketServer ss = (SocketServer)svc[i];
               String uri = ss.getProtocol()+"://"+ss.getHostname()+":"+ss.getPort()+"/";
               System.out.println("Service: '"+name+"' started @ "+uri);
            }
            else
               System.out.println("Service: '"+name+"' started");
         }
         else {
            System.err.println("\u0007ERROR: Could not start service '"+name+"'");
            ok = false;
         }
      }
      return ok;
   }

   public void stopServices() {
      Service[] svc = (Service[])services.toArray(EmptyServices);
      for ( int i = svc.length-1; i >= 0; i-- ) {
         String name = svc[i].getName();
         if ( svc[i].stop() == Service.RESULT_OK )
            System.out.println("Service: '"+name+"' stopped");
         else
            System.err.println("\u0007ERROR: Could not stop service '"+name+"'");
      }
   }

   public boolean addService(Service service) {
      String name = service.getName();
      if ( getService(name) != null ) {
         System.err.println("\u0007ERROR: Could not add service '"+name+"'");
         System.err.println("The service name duplicates one already in the service manager");
         return false;
      }
      if ( service.initialize() == Service.RESULT_OK ) {
         services.add(service);
         svcMap.put(name, service);
      }
      else {
         System.err.println("\u0007ERROR: Could not initialize service '"+name+"'");
         return false;
      }
      return true;
   }

   public boolean removeService(Service service) {
      if ( service.status() == Service.STATE_STARTED )
         service.stop();
      service.uninitialize();
      services.remove(service);
      svcMap.remove(service.getName());
      Kernel.disposeOf(service);
      return true;
   }

   public String[] listServices() {
      return (String[])svcMap.keySet().toArray(EmptyStrings);
   }

   public Service getService(String name) {
      return (Service)svcMap.get(name);
   }

   public void dispose() {
      Service[] svc = (Service[])services.toArray(EmptyServices);
      for ( int i = svc.length-1; i >= 0; i-- ) {
         svc[i].uninitialize();
         Kernel.disposeOf(svc[i]);
      }
   }
}


