package org.apache.xindice.util;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: StringUtilities.java,v 1.1.1.1 2001/12/06 19:33:58 bradford Exp $
 */

import java.net.*;
import java.util.*;

/**
 * StringUtilities provides a set of commonly used String parsing and
 * processing routines.
 */

public final class StringUtilities {
   private StringUtilities() {
   }

   /**
    * stringToInt quickly converts a String value to an int, returning
    * a default value if a conversion error occurs.
    *
    * @param value The value to convert
    * @param defvalue The default value to return
    * @return The converted value
    */
   public static int stringToInt(String value, int defvalue) {
      try {
         return Integer.parseInt(value);
      }
      catch ( Exception e ) {
         return defvalue;
      }
   }

   /**
    * stringToInt quickly converts a String value to an int, returning
    * 0 if a conversion error occurs.
    *
    * @param value The value to convert
    * @return The converted value
    */
   public static int stringToInt(String value) {
      return stringToInt(value, 0);
   }

   /**
    * leftJustify left-justifies a String value, space padding to width
    * characters if the string is less than width.
    *
    * @param value The value to left-justify
    * @param width The width to left-justify to
    * @return The left-justified value
    */
   public static String leftJustify(String value, int width) {
      StringBuffer sb = new StringBuffer(width);
      sb.append(value);
      for ( int i = sb.length(); i < width; i++ )
         sb.append(' ');
      if ( sb.length()>width )
         sb.setLength(width);
      return sb.toString();
   }

   /**
    * rightJustify right-justifies a String value, space padding to
    * width characters if the string is less than width.
    *
    * @param value The value to right-justify
    * @param width The width to right-justify to
    * @return The right-justified value
    */
   public static String rightJustify(String value, int width) {
      StringBuffer sb = new StringBuffer(width);
      sb.append(value);
      for ( int i = sb.length(); i < width; i++ )
         sb.insert(0, ' ');
      if ( sb.length()>width )
         return sb.toString().substring(sb.length()-width);
      else
         return sb.toString();
   }

   /**
    * parseBuffer parses through a String buffer and produces a
    * Map table using the provided delimiters.  This method can also
    * urldecode values and produce newline-delimited multi-value properties if
    * multiple values for the same key are detected in the buffer.
    *
    * @param retprops The Map to populate
    * @param inbuffer The String buffer to parse
    * @param delimiters The delimiters to use
    * @param urldecode Whether or not to URL-decode the values
    * @param multivalue Whether or not to parse multiple values
    */
   public static void parseBuffer(Map retprops, String inbuffer, String delimiters, boolean urldecode, boolean multivalue) {
      if ( inbuffer.length() == 0 )
         return;
      int idx = 0;
      String line;
      String key;
      String value;
      String prop;
      String temp;
      StringTokenizer st = new StringTokenizer(inbuffer, delimiters, false);
      while ( st.hasMoreTokens() ) {
         line = st.nextToken();
         idx = line.indexOf('=');
         if ( idx != -1 ) {
            key = line.substring(0, idx).trim();
            try {
               value = urldecode ? URLDecoder.decode(line.substring(idx+1).trim())
                                 : line.substring(idx+1).trim();
            }
            catch ( Exception e ) {
               value = line.substring(idx+1).trim();
            }
            if ( multivalue ) {
               temp = (String)retprops.get(key);
               if ( temp != null && temp.length() > 0 )
                  value = temp + "\u0001" + value;
               retprops.put(key, value);
            }
            else
               retprops.put(key, value);
         }
      }
   }

   /**
    * findWhiteSpace scans a String value for whitespace characters
    * from a specified starting position and returns the location of the first
    * whitespace character found.
    *
    * @param value The value to scan
    * @param start The starting position
    * @return The first whitespace location
    */
   public static int findWhiteSpace(String value, int start) {
      char[] chars = value.toCharArray();
      for ( int i = 0; i < chars.length; i++ )
         if ( chars[i] <= 32 )
            return i;
      return -1;
   }

   /**
    * findWhiteSpace scans a String value for whitespace characters
    * and returns the location of the first whitespace character found.
    *
    * @param value The value to scan
    * @return The first whitespace location
    */
   public static int findWhiteSpace(String value) {
      return findWhiteSpace(value, 0);
   }

   /**
    * javaEncode converts a String value to a Java-printable String.
    * All non-printable characters, such as newline and backspace are converted
    * to their respective escape sequences.  This method does not convert
    * UNICODE characters.
    *
    * @param value The value to convert
    * @return The converted value
    */
   public static String javaEncode(String value) {
      StringTokenizer st = new StringTokenizer(value, "\b\t\n\f\r\"\'\\", true);
      StringBuffer sb = new StringBuffer(value.length());
      String token;
      while ( st.hasMoreTokens() ) {
         token = st.nextToken();
         if ( token.equals("\b") )
            sb.append("\\b");
         else if ( token.equals("\t") )
            sb.append("\\t");
         if ( token.equals("\n") )
            sb.append("\\n");
         else if ( token.equals("\f") )
            sb.append("\\f");
         else if ( token.equals("\r") )
            sb.append("\\r");
         else if ( token.equals("\"") )
            sb.append("\\\"");
         else if ( token.equals("\'") )
            sb.append("\\\'");
         else if ( token.equals("\\") )
            sb.append("\\\\");
         else
            sb.append(token);
      }
      return sb.toString();
   }
}
